@@ -7,7 +7,7 @@ This script queries Jira, and generates a task-juggler input file to generate a
 import argparse
 import logging
 import re
-from abc import ABC, abstractmethod
+from abc import ABC
 from datetime import datetime, time
 from functools import cmp_to_key
 from getpass import getpass
@@ -19,13 +19,12 @@ from jira import JIRA, JIRAError
 from natsort import natsorted, ns
 
 DEFAULT_LOGLEVEL = 'warning'
-DEFAULT_JIRA_URL = 'https://jira.melexis.com/jira'
+DEFAULT_JIRA_URL = 'https://melexis.atlassian.net'
 DEFAULT_OUTPUT = 'jira_export.tjp'
 
 JIRA_PAGE_SIZE = 50
 
 TAB = ' ' * 4
-id_to_username_mapping = {}
 
 
 def fetch_credentials():
@@ -113,12 +112,50 @@ def calculate_weekends(date, workdays_passed, weeklymax):
 
 
 def to_username(value):
-    if isinstance(value, str) and len(value) >= 24:
-        if value not in id_to_username_mapping:
-            user = jirahandle.user(value)
-            id_to_username_mapping[value] = user.emailAddress.split('@')[0]
-        return id_to_username_mapping[value]
-    return value
+    """Converts the given value to a username (user ID), if needed, while caching the result.
+
+    Args:
+        value (str/jira.User): String (account ID or user ID) or User instance
+
+    Returns:
+        str: The corresponding username
+    """
+    user_id = value.accountId if hasattr(value, 'accountId') else str(value)
+    if user_id in id_to_username_mapping:
+        return id_to_username_mapping[user_id]
+
+    if not isinstance(value, str):
+        id_to_username_mapping[user_id] = determine_username(value)
+    elif len(value) >= 24:  # accountId
+        user = jirahandle.user(user_id)
+        id_to_username_mapping[user_id] = determine_username(user)
+    return id_to_username_mapping.get(user_id, value)
+
+
+def determine_username(user):
+    """Determines the username (user ID) for the given User.
+
+    Args:
+        user (jira.User): User instance
+
+    Returns
+        str: Corresponding username
+
+    Raises:
+        Exception: Failed to determine username
+    """
+    if getattr(user, 'emailAddress', ''):
+        username = user.emailAddress.split('@')[0]
+    elif getattr(user, 'name', ''):  # compatibility with Jira Server
+        username = user.name
+    elif getattr(user, 'displayName', ''):
+        full_name = user.displayName
+        username = f'"{full_name}"'
+        logging.error(f"Failed to fetch email address of {full_name!r}: they restricted its visibility; "
+                      f"using identifier {username!r} as fallback value.")
+    else:
+        raise Exception(f"Failed to determine username of {user}")
+    return username
 
 
 class JugglerTaskProperty(ABC):
@@ -144,7 +181,16 @@ class JugglerTaskProperty(ABC):
         if jira_issue:
             self.load_from_jira_issue(jira_issue)
 
-    @abstractmethod
+    @property
+    def is_empty(self):
+        """bool: True if the property contains an empty or uninitialized value"""
+        return not self.value or self.value == self.DEFAULT_VALUE
+
+    def clear(self):
+        """Sets the name and value to the default"""
+        self.name = self.DEFAULT_NAME
+        self.value = self.DEFAULT_VALUE
+
     def load_from_jira_issue(self, jira_issue):
         """Loads the object with data from a Jira issue
 
@@ -196,7 +242,9 @@ class JugglerTaskAllocate(JugglerTaskProperty):
                 for item in change.items:
                     if item.field.lower() == 'assignee':
                         if not before_resolved:
-                            self.value = to_username(getattr(item, 'from', None))
+                            self.value = getattr(item, 'from', None)
+                            if self.value:
+                                self.value = to_username(self.value)
                         else:
                             self.value = to_username(item.to)
                             return  # got last assignee before transition to Approved/Resolved status
@@ -205,12 +253,9 @@ class JugglerTaskAllocate(JugglerTaskProperty):
                         if self.value and self.value != self.DEFAULT_VALUE:
                             return  # assignee was changed after transition to Closed/Resolved status
 
-        if not self.value or self.value == self.DEFAULT_VALUE:
+        if self.is_empty:
             if getattr(jira_issue.fields, 'assignee', None):
-                if hasattr(jira_issue.fields.assignee, 'name'):
-                    self.value = jira_issue.fields.assignee.name
-                else:
-                    self.value = jira_issue.fields.assignee.emailAddress.split('@')[0]
+                self.value = to_username(jira_issue.fields.assignee)
             else:
                 self.value = self.DEFAULT_VALUE
 
@@ -345,6 +390,31 @@ class JugglerTaskDepends(JugglerTaskProperty):
         return ''
 
 
+class JugglerTaskTime(JugglerTaskProperty):
+    """Class for setting the start/end time of a juggler task"""
+
+    DEFAULT_VALUE = ''
+    PREFIX = ''
+
+    def validate(self, *_):
+        """Validates the current task property"""
+        if not self.is_empty:
+            valid_names = ('start', 'end')
+            if self.name not in valid_names:
+                raise ValueError(f'The name of {self.__class__.__name__} is invalid; expected a value in {valid_names}')
+
+    def __str__(self):
+        """Converts task property object to the task juggler syntax
+
+        Returns:
+            str: String representation of the task property in juggler syntax
+        """
+        if self.value:
+            return self.TEMPLATE.format(prop=self.name,
+                                        value=self.value)
+        return ''
+
+
 class JugglerTask:
     """Class for a task for Task-Juggler"""
 
@@ -385,6 +455,7 @@ task {id} "{description}" {{
         self.properties['allocate'] = JugglerTaskAllocate(jira_issue)
         self.properties['effort'] = JugglerTaskEffort(jira_issue)
         self.properties['depends'] = JugglerTaskDepends(jira_issue)
+        self.properties['time'] = JugglerTaskTime()
 
     def validate(self, tasks):
         """Validates (and corrects) the current task
@@ -463,6 +534,8 @@ class JiraJuggler:
             token (str): API token (or password)
             query (str): The query to run
         """
+        global id_to_username_mapping
+        id_to_username_mapping = {}
         logging.info('Jira endpoint: %s', endpoint)
 
         global jirahandle
@@ -528,7 +601,7 @@ class JiraJuggler:
         if not juggler_tasks:
             return None
         if output:
-            with open(output, 'w') as out:
+            with open(output, 'w', encoding='utf-8') as out:
                 for task in juggler_tasks:
                     out.write(str(task))
         return juggler_tasks
@@ -540,9 +613,9 @@ class JiraJuggler:
         If the task has been resolved, 'end' is added instead of 'depends' no matter what, followed by the
         date and time on which it's been resolved.
 
-        If it's the first unresolved task for a given assignee and it's not linked with 'depends on'/'is blocked by'
-        through JIRA, 'start' is added instead followed by the date and hour on which the task has been started,
-        i.e. current time minus time spent. For the other unresolved tasks, the effort estimate is 'Remaining' time
+        If it's the first unresolved task for a given assignee, 'start' is added followed by the date and hour on which
+        the task has been started, i.e. current time minus time spent.
+        For the other unresolved tasks, the effort estimate is 'Remaining' time
         only instead of 'Remaining + Logged' time since parallellism is not supported by
         TaskJuggler and this approach results in a more accurate forecast.
 
@@ -551,34 +624,34 @@ class JiraJuggler:
             weeklymax (float): Number of allocated workdays per week
             current_date (datetime.datetime): Offset-naive datetime to treat as the current date
         """
+        current_date_str = to_juggler_date(current_date)
         unresolved_tasks = {}
         for task in tasks:
             assignee = str(task.properties['allocate'])
-            if assignee not in unresolved_tasks:
-                unresolved_tasks[assignee] = []
 
             depends_property = task.properties['depends']
+            time_property = task.properties['time']
+
             if task.is_resolved:
-                depends_property.PREFIX = ''
-                depends_property.name = 'end'
-                depends_property.value = [task.resolved_at_repr]  # overwrite any links in JIRA
+                depends_property.clear()  # don't output any links in JIRA
+                time_property.name = 'end'
+                time_property.value = task.resolved_at_repr
             else:
-                if unresolved_tasks[assignee]:  # task with dependency
+                if assignee in unresolved_tasks:  # link to a preceding unresolved task
                     preceding_task = unresolved_tasks[assignee][-1]
                     depends_property.append_value(to_identifier(preceding_task.key))
-                elif not depends_property.value:  # first unresolved task for assignee
-                    depends_property.PREFIX = ''
-                    depends_property.name = 'start'
-                    val = to_juggler_date(current_date)
+                else:  # first unresolved task for assignee: set start time
+                    start_time = current_date_str
                     if task.issue.fields.timespent:
                         effort_property = task.properties['effort']
                         effort_property.value += task.issue.fields.timespent / JugglerTaskEffort.FACTOR
                         days_spent = task.issue.fields.timespent // 3600 / 8
                         weekends = calculate_weekends(current_date, days_spent, weeklymax)
                         days_per_weekend = min(2, 7 - weeklymax)
-                        val = f"%{{{val} - {days_spent + weekends * days_per_weekend}d}}"
-                    depends_property.append_value(val)
-                unresolved_tasks[assignee].append(task)
+                        start_time = f"%{{{start_time} - {days_spent + weekends * days_per_weekend}d}}"
+                    time_property.name = 'start'
+                    time_property.value = start_time
+                unresolved_tasks.setdefault(assignee, []).append(task)
 
     def sort_tasks_on_sprint(self, tasks, sprint_field_name):
         """Sorts given list of tasks based on the values of the field with the given name.
